from Tkinter import *
import sys, os, tkMessageBox, commands, py_compile
from tkFileDialog import askopenfilename
from time import localtime
import numpy as npy
from src import ABM_main

class abmGUI:
    def __init__(self, parent):
        """Build the GUI."""
        self.master = parent
        self.root_dir = os.getcwd()  # current working directory (for saving outputs)
        self.params = {}  # holds all Tkinter parameters
        self.attitudes = {1:"UNBIASED", 2:"RECKLESS", 3:"CAUTIOUS",4:"POOR", 5:"RICH"}
        # -------------
        clr = "#666666"
    
        
        # input ASCII GRID files
        Label(self.master, text='',width=45,relief=GROOVE).grid(row=0,padx=5, pady=5, sticky=W)
        Label(self.master, text='Land value file',fg=clr,width=15, anchor=W).grid(row=0,column=1)
        Button(self.master,text='Open', width=15, command=self.lv_file_open).grid(row=0,column=2)
        self.params['file_land_value']=StringVar(); self.params['file_land_value'].set('')
        
        Label(self.master, text='',width=45,relief=GROOVE).grid(row=1,padx=5, pady=5, sticky=W)
        Label(self.master, text='Scenic beauty file',fg=clr,width=15, anchor=W).grid(row=1,column=1)
        Button(self.master,text ='Open',width=15, command = self.sb_file_open).grid(row=1,column=2)
        self.params['file_scenic_beauty']=StringVar(); self.params['file_scenic_beauty'].set('')       

        # preference sliders
        slider_frame = LabelFrame(self.master, text="Preference for land value",fg=clr)
        slider_frame.grid(row=2, sticky=W, padx=5)
    
        self.params['Agent1_land_val_weight'] = DoubleVar(); self.params['Agent1_land_val_weight'].set(0.0)
        self.slider(slider_frame, self.params['Agent1_land_val_weight'], 0, 1, 'Agent [1]',310)
        self.params['Agent2_land_val_weight'] = DoubleVar(); self.params['Agent2_land_val_weight'].set(0.5)
        self.slider(slider_frame, self.params['Agent2_land_val_weight'], 0, 1, 'Agent [2]',310)        
        self.params['Agent3_land_val_weight'] = DoubleVar(); self.params['Agent3_land_val_weight'].set(1.0)
        self.slider(slider_frame, self.params['Agent3_land_val_weight'], 0, 1, 'Agent [3]',310)

        # attitude option menus
        option_frame = LabelFrame(self.master,text="Attitude to risk",fg=clr)
        option_frame.grid(row=2,column=1,columnspan=2,sticky=E+N, padx=5)

        auf_labels = self.attitudes.values()
        Label(option_frame,text='Agent [1]',width=35, anchor=W,pady=14).pack(side='top')
        self.params['Agent1_attitude'] = StringVar(); self.params['Agent1_attitude'].set(auf_labels[0])
        self.optionmenu(option_frame, self.params['Agent1_attitude'],auf_labels)
        Label(option_frame,text='Agent [2]',width=35, anchor=W,pady=14).pack(side='top')
        self.params['Agent2_attitude'] = StringVar(); self.params['Agent2_attitude'].set(auf_labels[1])
        self.optionmenu(option_frame, self.params['Agent2_attitude'],auf_labels)
        Label(option_frame,text='Agent [3]',width=35, anchor=W,pady=14).pack(side='top')
        self.params['Agent3_attitude'] = StringVar(); self.params['Agent3_attitude'].set(auf_labels[2])
        self.optionmenu(option_frame, self.params['Agent3_attitude'],auf_labels)

        # feedback sliders
        feed_frame = LabelFrame(self.master, text="Feedbacks", fg=clr)
        feed_frame.grid(row=3,sticky=W, padx=5, pady=5)
        self.params['feedback_decrease_beauty'] = DoubleVar(); self.params['feedback_decrease_beauty'].set(0.01)
        self.slider(feed_frame, self.params['feedback_decrease_beauty'], 0.0, 0.1, 'Decrease beauty', 310)
        self.params['feedback_increase_value'] = DoubleVar(); self.params['feedback_increase_value'].set(0.01)
        self.slider(feed_frame, self.params['feedback_increase_value'], 0.0, 0.1, 'Increase value', 310)

        # model-specific text entries
        time_frame = LabelFrame(self.master,text="Model",fg=clr)
        time_frame.grid(row=3,column=1,columnspan=2,sticky=N,padx=5, pady=5)
        self.params['ticks'] = IntVar(); self.params['ticks'].set(15)
        self.textentry(time_frame, self.params['ticks'], 'Number of ticks'+" "*14, row = 0)
        self.params['seed'] = IntVar(); self.params['seed'].set(9999)
        self.textentry(time_frame, self.params['seed'], 'Random seed    '+" "*14, row = 1, state=DISABLED)
        # Execution button
        self.sim = Button(time_frame, text='Execute ABM',width=15, state=DISABLED,command=self.execute)
        self.sim.grid(row=3,column=0,sticky=S+W,pady=2,padx=1)
        # Quit button
        Button(time_frame, text='Quit',width=15,command=self.quit).grid(row=3,column=1,sticky=S+E,pady=2, padx=1)
        
        # press q on the keyboard for exit:
        self.master.bind('<q>', self.quit)


    def lv_file_open(self):        
        lv_file = askopenfilename(title="Open land value file",filetypes=[("Text ASCII GRID","*.asc")])#,("Text GRID","*.txt")])
        Label(self.master,text=lv_file[-39:],fg="blue",width=45,relief=GROOVE,anchor=W).grid(row=0,padx=5, pady=5, sticky=W)
        self.params['file_land_value'].set(lv_file)
        cur_lv = self.params['file_land_value'].get()
        cur_sb = self.params['file_scenic_beauty'].get()
        if len(cur_lv) > 0 and  len(cur_sb) > 0:
            self.sim.config(state = NORMAL,bg="#CCFF99", fg="#006600")

    def sb_file_open(self):
        sb_file = askopenfilename(title="Open scenic beauty file",filetypes=[("Text ASCII GRID","*.asc")])#,("Text GRID","*.txt")])
        Label(self.master,text=sb_file[-39:],fg="blue",width=45,relief=GROOVE,anchor=W).grid(row=1,padx=5, pady=5, sticky=W)
        self.params['file_scenic_beauty'].set(sb_file)
        cur_lv = self.params['file_land_value'].get()
        cur_sb = self.params['file_scenic_beauty'].get()
        if len(cur_lv) > 0 and  len(cur_sb) > 0:
            self.sim.config(state = NORMAL, bg="#CCFF99", fg="#006600")


    def quit(self,event=None):
        """Destroy GUI; called from Quit button or 'q' binding."""
        self.master.destroy()

    def optionmenu(self, parent,variable,values):
        """Make an option menu tied to variable"""
        widget = OptionMenu(parent,variable,*values)
        widget.pack(side='top',fill=X)
        return widget

    def slider(self, parent, variable, low, high, label, length):
        """Make a slider [low,high] tied to variable"""
        widget = Scale(parent, orient='horizontal',
          from_=low, to=high,
          # tickmarks on the slider "axis":
          tickinterval=(high-low)/4.0,
          # the steps of the counter above the slider:
          resolution=(high-low)/100.0,
          label=label,length=length,variable=variable)  # slider value tied to variable
        widget.pack(side='top')
        return widget

    def textentry(self, parent, variable, label, row, state=NORMAL):
        """Make a textentry field tied to variable"""
        # pack a label and entry horizontally in a frame
        l = Label(parent, text=label)
        l.grid(column=0, row=row, sticky=W+E)
        widget = Entry(parent, textvariable=variable, width=5, state=state)
        widget.grid(column=1, row=row, padx=10, pady=22,sticky=W+E)
        return widget

    def configparams(self):
        """Check and save input parameter values"""
        outdir = self.root_dir+"/results"
        if not os.path.exists(outdir):
            os.mkdir(outdir)
        # FIXED MODEL PARAMS
##        sample_size = 0.1 # fixed in agent.py
##        self.params['sample_size'] = DoubleVar(); self.params['sample_size'].set(sample_size)
        convert = 0.2 # NOTE: the % of land converted to developed after the simulation is FIXED
        # calculate the demands per agent
        cur_lv = self.params['file_land_value'].get()
        try:
            size_lv = npy.loadtxt(cur_lv, dtype=float).shape
        except:
            tkMessageBox.Message(icon='error', type='ok',
                                 message='Invalid format for land value grid, closing application',
                                 title='Critical error').show()
            sys.exit(1)
        cur_sb = self.params['file_scenic_beauty'].get()
        try:
            size_sb = npy.loadtxt(cur_sb, dtype=float).shape
        except:
            tkMessageBox.Message(icon='error', type='ok',
                                 message='Invalid format for scenic beauty grid, closing application',
                                 title='Critical error').show()
            sys.exit(1)
        if size_lv != size_sb:
            tkMessageBox.Message(icon='error', type='ok',
                                 message='Land use grid and scenic beauty grid have different sizes, closing application',
                                 title='Critical error').show()
            sys.exit(1)
        cellnum = size_sb[0]*size_sb[1]
        convert_cell_num = cellnum*convert
        ticks = None
        try: # check 'tick' entry
            ticks = self.params['ticks'].get()
        except:
            tkMessageBox.Message(icon='warning', type='ok',
                                 message='Invalid input for ticks, setting to default: 15',
                                 title='Input error').show()
            self.params['ticks'].set(15)
            ticks = self.params['ticks'].get()
        per_agent_per_tick = int(convert_cell_num/3.0/ticks)
        if per_agent_per_tick < 1:
            per_agent_per_tick = 1
            tkMessageBox.Message(icon='warning', type='ok',
                                 message='Demand per agent per tick less than 1, setting to default: 1',
                                 title='Input error').show()
        # FUTURE: assert that there is enough land available for development, if not, display critical error & quit
        # add demand to parameters
        self.params['demand_per_agent'] = IntVar(); self.params['demand_per_agent'].set(per_agent_per_tick)
        # check 'seed' entry
        try: # check 'seed' entry
            seed = self.params['seed'].get()
        except:
            tkMessageBox.Message(icon='warning', type='ok',
                                 message='Invalid input for random seed, setting to default: 9999',
                                 title='Input error').show()
            self.params['seed'].set(9999)
        # create output folder
        datetime_stamp = '%4d-%02d-%02dT%02d-%02d-%02d' % localtime()[:6]
        outdir_run = outdir+"/"+datetime_stamp
        os.mkdir(outdir_run); os.chdir(outdir_run)
        # add datetime_stamp to parameters
        self.params['datetime_stamp'] = StringVar(); self.params['datetime_stamp'].set(datetime_stamp)
        paramfile = '%s-%s.%s' % ('ABM-inputs',datetime_stamp,'txt')
        content = []
        for i in self.params.items():
            val = i[1].get()
            content.append((i[0],str(val)))
        content.sort()
        content_str = ""
        for line in content:
            content_str += ",".join(line)+'\n'
        f=open(paramfile, 'w')
        f.write(content_str)
        f.close()
        print paramfile,"saved in"
        print os.getcwd()  # This can be also done as a message box
        os.chdir(self.root_dir)

    def execute(self):
        """Run ABM_main.py"""
        self.configparams()
##        try:
        ABM_main.main( Ag1_auf=self.params['Agent1_attitude'].get(),
                       Ag2_auf=self.params['Agent2_attitude'].get(),
                       Ag3_auf=self.params['Agent3_attitude'].get(),
                       Ag1_lval_w=self.params['Agent1_land_val_weight'].get(),
                       Ag2_lval_w=self.params['Agent2_land_val_weight'].get(),
                       Ag3_lval_w=self.params['Agent3_land_val_weight'].get(),
                       demand=self.params['demand_per_agent'].get(),
                       feedback_dec_beauty=self.params['feedback_decrease_beauty'].get(),
                       feedback_inc_value=self.params['feedback_increase_value'].get(),
                       file_lval=self.params['file_land_value'].get(),
                       file_scby=self.params['file_scenic_beauty'].get(),
                       seed=self.params['seed'].get(),
                       ticks=self.params['ticks'].get(),
                       datetime_s=self.params['datetime_stamp'].get() )
##            
##        except:
##            tkMessageBox.Message(icon='error', type='ok',message='Underlying ABM_main.py script failed',\
##                                 title='Error').show()
##            sys.exit(1)
# to hide the command window, use .pyc not .py
#------------
# execute GUI
root = Tk() # main window
root.title('Agent-Based Model of Residential Development Input')
ws = str(int((root.winfo_screenwidth()-600)*0.7))
root.geometry("600x500+"+ws+"+10")
root.resizable(width=FALSE, height=FALSE)
simviz = abmGUI(root)
root.mainloop()
