# THE MAIN CODE FOR THE ABM OF RESIDENTIAL DEVELOPERS
# simplified version with feedbacks (as in IJGIS paper on GSA)
# Arika Ligmann-Zielinska, 2010
# version
#   for two layers (scenic beauty and land value), and
#   for three developers
# ----------------------------------------------------
import os,random,sys,shutil, math
import numpy
import space, agent

# ------------------------------------------------------------------------

# default values of input parameters
Agent1_attitude='UNBIASED'; Agent1_land_val_weight=0.0
Agent2_attitude='RECKLESS'; Agent2_land_val_weight=0.5
Agent3_attitude='CAUTIOUS'; Agent3_land_val_weight=1.0
demand_per_agent=9; feedback_decrease_beauty=0.01
feedback_increase_value=0.01
file_land_value=os.path.dirname(os.getcwd())+'/data/land_value.asc'
file_scenic_beauty=os.path.dirname(os.getcwd())+'/data/scenic_beauty_0.asc'
aseed=9999; theticks=15
datetime_stamp = "unspecified"+str(random.randint(1000,9999))
# -------------------------------------------------------------------------
# read variables from the command line, one by one


def main(Ag1_auf=Agent1_attitude,
         Ag2_auf=Agent2_attitude,
         Ag3_auf=Agent3_attitude,
         Ag1_lval_w=Agent1_land_val_weight,
         Ag2_lval_w=Agent2_land_val_weight,
         Ag3_lval_w=Agent3_land_val_weight,
         demand=demand_per_agent,
         feedback_dec_beauty=feedback_decrease_beauty,
         feedback_inc_value=feedback_increase_value,
         file_lval=file_land_value,
         file_scby=file_scenic_beauty,
         seed=aseed,
         ticks=theticks,
         datetime_s=datetime_stamp):
    """executes ABM"""
    # output directories
    root = os.getcwd()
    resultdir = root+"/results/"+datetime_s
    if not os.path.exists(resultdir):
        os.mkdir(resultdir)
    mapsdir = resultdir+"/maps"; os.mkdir(mapsdir)
    lusedir = mapsdir+"/landuse"; os.mkdir(lusedir)
    lvaldir = mapsdir+"/landvalue"; os.mkdir(lvaldir)
    scbtdir = mapsdir+"/scenicbeauty"; os.mkdir(scbtdir)

    # ENVIRONMENT
    grid = space.Space(file_lval,file_scby)
    random.seed(seed)
    # AGENTS
    developers = []
    # construct the developers
    developerAgent1 = agent.Developer(Ag1_auf,demand,Ag1_lval_w,grid.shape)
    developers.append(developerAgent1)
    developerAgent2 = agent.Developer(Ag2_auf,demand,Ag2_lval_w,grid.shape)
    developers.append(developerAgent2)
    developerAgent3 = agent.Developer(Ag3_auf,demand,Ag3_lval_w,grid.shape)
    developers.append(developerAgent3) 
    # MODEL EXECUTION
    outstats="TICK,AGENT1_DISUTILITY,AGENT2_DISUTILITY,AGENT3_DISUTILITY,TOTAL_DISUTILITY,AI\n"
    for t in range(ticks):
        newrecord = str(t+1)+','
        # MARKET ASSESSMENT
        for developer in developers:
            developer.scoreSites(grid)
            developer.createInvestmentSet(grid)
            developer.createInvestmentPlan()
        # BIDDING
        bidding = True
        while bidding:
            currentbids = agent.bids(developers) # best ranks
            currentties = agent.ties(developers,currentbids)
            agent.buy(developers,currentbids,currentties) # sell based on best rank
            # update landuse
            grid.develop(developers)
            demandNotMet = agent.stillneeded(developers)
            if demandNotMet == 0:
                # bidding finished
                bidding = False
                for developer in developers:
                    developer.clear()
            else:
                # delete purchased land from consideration, and continue bidding
                for developer in developers:
                    developer.updateInvestmentSet(grid)
                    developer.createInvestmentPlan()
        # FEEDBACK TO ENVIRONMENT
        grid.decrease_beauty(feedback_dec_beauty)
        grid.increase_landvalue(feedback_inc_value)
        # CALCULATE STATISTICS
        total = 0
        for developer in developers:
            total += developer.dissattisfaction
            newrecord += str(developer.dissattisfaction)+','
        newrecord += str(total)+','
        AI = grid.AI()
        newrecord += str(AI)+'\n'
        outstats+=newrecord
        # SAVE OUTPUT
        #land use
        fname = "%s/%03d%s" % (lusedir, t+1, "_luse.asc")
        space.save(grid.landuse,fname)
        #scenic beauty
        fname = "%s/%03d%s" % (scbtdir, t+1, "_sbty.asc")
        space.save(grid.beauty,fname,format = 'f')
        #land value
        fname = "%s/%03d%s" % (lvaldir, t+1, "_lval.asc")
        space.save(grid.landvalue,fname,format = 'f')
        print "completed:",round((t+1)/float(ticks)*100,1),"%"
    statsfile = resultdir+"/"+datetime_s+"_output_stats.csv"
    f = open(statsfile,'w')
    f.write(outstats)
    f.close()
    print "Saved", statsfile
    print "FINISHED; You can run the model with different inputs\n\n"
##        total_developed="developed: "+str(numpy.sum(grid.landuse))
##        print total_developed

