# auf.py ATTITUDE UTILITY FUNCTIONS
# Arika Ligmann-Zielinska, 2007-2008
# updated to numpy
# ***************************************************
import numpy

def unbiased(criterion):
    """ This function recalculates input value
        according to UNBIASED Attitude Utility Function.
    """
    prospect = criterion
    return prospect
    
def reckless(criterion, skewness=3):
    """ This function recalculates input value
        according to RECKLESS Attitude Utility Function.
    """
    prospect = (numpy.exp(skewness*criterion)-1)/skewness
    return prospect

def cautious(criterion, skewness=3):
    """ This function recalculates input value
        according to CAUTIOUS Attitude Utility Function.
    """
    # function is asymptotic to -1/3
    criterion = numpy.where(criterion < -0.33, -0.33, criterion)
    prospect = numpy.log(skewness*criterion + 1)/skewness
    return prospect

def poor(criterion, skewness=3):
    """ This function recalculates input value
        according to POOR Attitude Utility Function.
    """
    prospect = skewness*numpy.power(criterion, skewness)
    return prospect

def rich(criterion, skewness=3):
    """ This function recalculates input value
        according to RICH Attitude Utility Function.
    """
    positive = numpy.power(criterion*(1.0/skewness), 1.0/skewness)
    negative = -numpy.power(-criterion*(1.0/skewness), 1.0/skewness)
    prospect = numpy.where(criterion >= 0, positive, negative)
    return prospect
