# converts all text grids in input folder to .png images
# Arika Ligmann-Zielinska, MSU, 2008
# -------------------------------------------------------
import numpy, glob
from pylab import *
from matplotlib.ticker import MultipleLocator, FormatStrFormatter
import Tkinter, tkFileDialog

# -------------------------------------------------------
# GLOBAL CONSTANTS
colorramp=cm.Blues       # color can be found in _cm.py
#e.g.
#          jet,autumn,gray,hsv,spring,summer,winter,
#          Spectral,Blues,Reds,Greens,RdGy
# COMMENT: if the range of data is large with few outliers
#          the coloring does not reflect all values
#          e.g. for the case where luses 0,1,2,..., and no data -9999
#          then two colors only emerge, solution:
#          change no data to smaller number like -1
saveimage = True
gridlines = False
# -> should not use gridlines and ticks for large grids
# -------------------------------------------------------
import os, sys

def load(infile):
    """ This function returns an array of data of
        input txt grid-like file.
    """
    f = open(infile, 'r')
    if f.read(5)=='ncols':
        data = f.readlines()
        f.close()
        data = data[6:]
        grid = []
        for line in data:
            row = map(float,line.strip().split())
            grid.append(row)
        outarray = numpy.array(grid,float)
    else:
        f.close()
        outarray=numpy.loadtxt(infile, dtype = float)
    return outarray

    
# ----- application -------------------------------------

print "-----------------------------------------"
print "to PNG converter                         "
print
print "Arika Ligmann-Zielinska, MSU, 2008       "
print "-----------------------------------------"

# ----- input -------------------------------------------

root = Tkinter.Tk()
root.withdraw()
path = tkFileDialog.askdirectory(parent=root,title='Please select a directory with *.asc files')
path +="/"

# -------------------------------------------------------
outdir = path+"/images/"
if not os.path.exists(outdir):
    os.mkdir(outdir)
os.chdir(outdir)
for f in os.listdir(path):
    if f[-4:] == ".asc":
        griddata=load(path+f)
    else: continue
    print "File name:",os.path.basename(f)
    print "Min:",numpy.min(griddata)
    print "Max:",numpy.max(griddata)
    print "Size:",griddata.shape

    # display
    ax = subplot(111)
    ax.grid(gridlines) # show/hide gridlines

    if gridlines is True:
        cellwidth = MultipleLocator(1) # cell size is 1
        format = FormatStrFormatter('%d') # ticks as integers
        # Locate gridline ticks
        ax.xaxis.set_major_locator(cellwidth)
        ax.xaxis.set_major_formatter(format)
        ax.yaxis.set_major_locator(cellwidth)
        ax.yaxis.set_major_formatter(format)    
    else:
        ax.xaxis.set_major_locator(NullLocator())
        ax.yaxis.set_major_locator(NullLocator())

    im = imshow(griddata, interpolation='nearest', cmap=colorramp, \
                extent=[0,griddata.shape[0],0,griddata.shape[1]])
    # save image
    if saveimage is True:
        savefig(f[:-4])
    # show()
root.destroy()
