# dynamic grid display

# Arika Ligmann-Zielinska, 2010
# IMPORTs ----------------------------------------------------------

import matplotlib, wx, os, sys
import numpy
matplotlib.use('WXAgg')

from matplotlib.cm import *
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg
from matplotlib.figure import Figure


# CLASS DEFINITIONs -------------------------------------------------

class GridPlot(wx.Frame):
    # customized wx Frame (-> GUI window)
    
    def __init__(self,ini_file,title="Dynamic Grid Viewer",
                 updateTime=200, colorMap=Paired):
        wx.Frame.__init__(self, None, title=title)
        self.figure = Figure()
        self.canvas = FigureCanvasWxAgg(self, -1,self.figure)
        self.colormap = colorMap
        self.timer = wx.Timer(self)
        self.timer.Start(updateTime) # higher = slower
        self.Fit() # resizes display on window resize
        outarray=numpy.loadtxt(ini_file, dtype = float)
        self.grid = outarray
        
        # event binders:
        self.Bind(wx.EVT_TIMER, self.onTimer) # updates display
        self.Bind(wx.EVT_CLOSE, self.onClose) # closes application

    def initialize(self):
        mapExtent = [0,self.grid.shape[1],0,self.grid.shape[0]]
        plot = self.figure.add_subplot(111) # default style
        # draws image:
        self.image = plot.imshow(self.grid,cmap=self.colormap,\
                                 interpolation='nearest',\
                                 extent=mapExtent)

        
    def onTimer(self,evt):
        global file_counter
        global files_to_redner
        infile = files_to_render[file_counter]
        self.grid=numpy.loadtxt(infile, dtype = float)
        self.image.set_array(self.grid)
        self.canvas.draw()
        if file_counter < len(files_to_render)-1:
            file_counter += 1

    def onClose(self,evt):
        self.timer.Stop()
        evt.Skip()

# APPLICATION ----------------------------------------------------------------
if __name__ == '__main__':
    app = wx.PySimpleApp()
    root = ""
    cwdir = os.getcwd()
    dirbox = wx.DirDialog(None,message="Select directory with maps to render:",
                          defaultPath = cwdir,style=wx.DD_DIR_MUST_EXIST)
    if dirbox.ShowModal() == wx.ID_OK:
        root = dirbox.GetPath()
        print "Dir selected:",root
    else:
        sys.exit(1)
    dirbox.Destroy()
    files = os.listdir(root)
    files.sort()
    files_to_render = []
    for f in files:
        if f[-4:] == ".asc":
            apath = root+'/'+f
            files_to_render.append(apath)
    if len(files_to_render)==0:
         dlg = wx.MessageDialog(None, message='The folder does not contain ASCII GRID files',
                               caption='Error', style = wx.OK | wx.ICON_ERROR)
         dlg.ShowModal()
         dlg.Destroy()
         sys.exit(1)
    ini_map = files_to_render.pop(0)
    file_counter = 0
    frame = GridPlot(ini_map,colorMap=gist_earth_r)
    frame.initialize()
    frame.Show()
    app.MainLoop()
    
