# Raster viewer
# Displays input file and optionally saves as a JPG image
# Takes any of the following as inputs:
#       -> ASCII grid (text format with header)
#       -> simple txt file with rows and columns
# Arika Ligmann-Zielinska, MSU, 2008
# -------------------------------------------------------
import Tkinter, tkFileDialog
import sys
import numpy
from pylab import *
from matplotlib.ticker import MultipleLocator, FormatStrFormatter

# -------------------------------------------------------
# GLOBAL CONSTANTS
colorramp=cm.Blues       # color can be found in _cm.py
#e.g.
#          jet,autumn,gray,hsv,spring,summer,winter,
#          Spectral,Blues,Reds,Greens,RdGy
# COMMENT: if the range of data is large with few outliers
#          the coloring does not reflect all values
#          e.g. for the case where luses 0,1,2,..., and no data -9999
#          then two colors only emerge, solution:
#          change no data to smaller number like -1
saveimage = True
gridlines = False
# -> should not use gridlines and ticks for large grids
# -------------------------------------------------------

def openbox():
    """opens dialog box and returns the user-selected file"""
    root  = Tkinter.Tk()
    root.withdraw()
    infile = ""
    f1 = tkFileDialog.askopenfile()
    if f1.name[-4:] == ".txt" or f1.name[-4:] == ".asc":
        infile = f1.name
    else:
        print "ERROR: Invalid data format"
        sys.exit(1)
    f1.close()
    return infile

def load(infile):
    """ This function returns an array of data of
        input txt grid-like file.
    """
    f = open(infile, 'r')
    if f.read(5)=='ncols':
        data = f.readlines()
        f.close()
        data = data[6:]
        grid = []
        for line in data:
            row = map(float,line.strip().split())
            grid.append(row)
        outarray = numpy.array(grid,float)
    else:
        f.close()
        outarray=numpy.loadtxt(infile, dtype = float)
    return outarray

    
# ----- application -------------------------------------

print "-----------------------------------------"
print "Grid viewer                              "
print
print "Arika Ligmann-Zielinska, MSU, 2008       "
print "-----------------------------------------"

print
print
print "Allowed input files (txt only)           "
print "         1. ASCII grid (with header)     "
print "         2. simple txt (rows and cols)   "
print


# read data
gridfile=openbox()
griddata=load(gridfile)

print "grid name:",gridfile
print "extent:",griddata.shape[0],"rows",griddata.shape[1],"cols"
print "min (WHITES):", numpy.min(griddata)
print "max (BLUES):",numpy.max(griddata)
# display
ax = subplot(111)
ax.grid(gridlines) # show/hide gridlines

if gridlines is True:
    cellwidth = MultipleLocator(1) # cell size is 1
    format = FormatStrFormatter('%d') # ticks as integers
    # Locate gridline ticks
    ax.xaxis.set_major_locator(cellwidth)
    ax.xaxis.set_major_formatter(format)
    ax.yaxis.set_major_locator(cellwidth)
    ax.yaxis.set_major_formatter(format)    
else:
    ax.xaxis.set_major_locator(NullLocator())
    ax.yaxis.set_major_locator(NullLocator())


im = imshow(griddata, interpolation='nearest', cmap=colorramp, \
            extent=[0,griddata.shape[0],0,griddata.shape[1]])

# save image
if saveimage is True:
    savefig(gridfile[:-4])

show()
